package labss.lang.il.visitors;

import java.io.PrintWriter;

import labss.lang.il.model.Addition;
import labss.lang.il.model.And;
import labss.lang.il.model.AssignmentStatement;
import labss.lang.il.model.BlockStatement;
import labss.lang.il.model.BooleanLiteral;
import labss.lang.il.model.Division;
import labss.lang.il.model.Equals;
import labss.lang.il.model.GreaterThan;
import labss.lang.il.model.IfElseStatement;
import labss.lang.il.model.IfStatement;
import labss.lang.il.model.IntLiteral;
import labss.lang.il.model.LessThan;
import labss.lang.il.model.Multiplication;
import labss.lang.il.model.Not;
import labss.lang.il.model.Or;
import labss.lang.il.model.Statement;
import labss.lang.il.model.Subtraction;
import labss.lang.il.model.Variable;
import labss.lang.il.model.WhileStatement;
import labss.lang.il.reflect.ILEntityDescriptorEnum;

import org.whole.lang.matchers.Matcher;
import org.whole.lang.operations.PrettyPrinterOperation;

public class ILPrettyPrinterVisitor extends ILIdentityVisitor {
	protected final PrintWriter out;

	public ILPrettyPrinterVisitor(PrettyPrinterOperation op) {
		wSetOperation(op);
    	out = op.getPrintWriter();
	}

	protected final PrettyPrinterOperation getOperation() {
		return (PrettyPrinterOperation) wGetOperation();
	}
	protected final void setRelativeIndentation(int delta) {
		getOperation().setRelativeIndentation(delta);
	}
	protected final void printIndentation() {
		getOperation().printIndentation();
	}

	public void visit(BlockStatement stm) {
		for (int i=0; i<stm.wSize(); i++)
			((Statement) stm.wGet(i)).accept(this);
	}
	public void visit(WhileStatement entity) {
		printIndentation();
		out.print("while (");
		entity.getCondition().accept(this);
		out.print(')');

		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getTrueBody()))
			out.println(" {");
		else
			out.println();

		setRelativeIndentation(+1);
		entity.getTrueBody().accept(this);
		setRelativeIndentation(-1);

		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getTrueBody())) {
			printIndentation();
			out.println("}");
		}
	}
	public void visit(IfStatement entity) {
		printIndentation();
		out.print("if (");
		entity.getCondition().accept(this);
		out.print(')');

		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getTrueBody()))
			out.println(" {");
		else
			out.println();

		setRelativeIndentation(+1);
		entity.getTrueBody().accept(this);
		setRelativeIndentation(-1);
		
		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getTrueBody())) {
			printIndentation();
			out.println("}");
		}
	}
	public void visit(IfElseStatement entity) {
		printIndentation();
		out.print("if (");
		entity.getCondition().accept(this);
		out.print(')');

		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getTrueBody()))
			out.println(" {");
		else
			out.println();

		setRelativeIndentation(+1);
		entity.getTrueBody().accept(this);
		setRelativeIndentation(-1);

		printIndentation();
		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getTrueBody()))
			out.print("} ");

		out.print("else");

		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getFalseBody()))
			out.println(" {");
		else
			out.println();

		setRelativeIndentation(+1);
		entity.getFalseBody().accept(this);
		setRelativeIndentation(-1);
		
		if (Matcher.matchImpl(ILEntityDescriptorEnum.BlockStatement, entity.getFalseBody())) {
			printIndentation();
			out.println("}");
		}
	}
	public void visit(AssignmentStatement entity) {
		printIndentation();
		entity.getVariable().accept(this);
		out.print(" = ");
		entity.getExpression().accept(this);
		out.println(";");
	}

	public void visit(BooleanLiteral entity) {
		out.print(entity.wBooleanValue());
	}

	public void visit(IntLiteral entity) {
		out.print(entity.wIntValue());
	}

	public void visit(Variable entity) {
		out.print(entity.wStringValue());
	}

	public void visit(Addition entity) {
		entity.getExp1().accept(this);
		out.print(" + ");
		entity.getExp2().accept(this);
	}

	public void visit(Subtraction entity) {
		entity.getExp1().accept(this);
		out.print(" - ");
		entity.getExp2().accept(this);
	}

	public void visit(Multiplication entity) {
		entity.getExp1().accept(this);
		out.print(" * ");
		entity.getExp2().accept(this);
	}

	public void visit(Division entity) {
		entity.getExp1().accept(this);
		out.print(" / ");
		entity.getExp2().accept(this);
	}

	public void visit(And entity) {
		entity.getExp1().accept(this);
		out.print(" && ");
		entity.getExp2().accept(this);
	}

	public void visit(Or entity) {
		entity.getExp1().accept(this);
		out.print(" || ");
		entity.getExp2().accept(this);
	}

	public void visit(Not entity) {
		out.print("!");
		entity.getExp().accept(this);
	}

	public void visit(Equals entity) {
		entity.getExp1().accept(this);
		out.print(" == ");
		entity.getExp2().accept(this);
	}

	public void visit(LessThan entity) {
		entity.getExp1().accept(this);
		out.print(" < ");
		entity.getExp2().accept(this);
	}

	public void visit(GreaterThan entity) {
		entity.getExp1().accept(this);
		out.print(" > ");
		entity.getExp2().accept(this);
	}
}
