package labss.lang.motomondiale;

import java.io.PrintStream;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import labss.lang.motomondiale.factories.MotoMondialeEntityFactory;
import labss.lang.motomondiale.model.Classifica;
import labss.lang.motomondiale.model.Classificato;
import labss.lang.motomondiale.model.Gara;
import labss.lang.motomondiale.model.IMotoMondialeEntity;
import labss.lang.motomondiale.model.MotoMondiale;
import labss.lang.motomondiale.model.Nome;
import labss.lang.motomondiale.model.Pilota;
import labss.lang.motomondiale.model.Team;
import labss.lang.motomondiale.visitors.MotoMondialeTraverseAllVisitor;

public class GestioneMotoMondiale {
	public static void ricalcolaClassificaPiloti(MotoMondiale mm) {
		MotoMondialeEntityFactory ef = MotoMondialeEntityFactory.instance;
		
		mm.setClassifiche(ef.createClassifiche());

		Classifica ca = ef.createClassifica();
		ca.setNome(ef.createNome("ClassificaPiloti"));
		CompilaClassificaPilotiVisitor ccpv = new CompilaClassificaPilotiVisitor();
		mm.getCalendario().accept(ccpv);
		for (Classificato co : ccpv.getClassificati())
			ca.getClassificati().wAdd(co);
		mm.getClassifiche().wAdd(ca);
	}
	public static void stampaClassifiche(MotoMondiale mm) {
		mm.accept(new StampaClassificheVisitor(System.out));
	}
	public static void validaPiloti(MotoMondiale mm) {
		mm.accept(new ValidaPilotiVisitor(DerivaMappaPilotiVisitor.getMappaPiloti(mm), System.out));
	}

	private static class CompilaClassificaPilotiVisitor extends MotoMondialeTraverseAllVisitor {
		private Map<String, Classificato> classificatiMap = new HashMap<String, Classificato>();
		private static final int[] punteggio = new int[] {
				25, 20, 16, 13, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1};
		private int posizioneArrivo;

		public SortedSet<Classificato> getClassificati() {
			SortedSet<Classificato> classificati = new TreeSet<Classificato>(new Comparator<Classificato>() {
				public int compare(Classificato o1, Classificato o2) {
					return o2.getPunti().wIntValue() - o1.getPunti().wIntValue();
				}
			});
			classificati.addAll(classificatiMap.values());
			return classificati;
		}

		@Override
		public void visit(Gara entity) {
			posizioneArrivo = 0;
			entity.getOrdineDArrivo().accept(visitor1);
		}
		@Override
		public void visit(Nome entity) {
			String pilota = entity.wStringValue();
			Classificato c = classificatiMap.get(pilota);
			if (c == null) {
				MotoMondialeEntityFactory ef = MotoMondialeEntityFactory.instance;
				classificatiMap.put(pilota, c = ef.createClassificato(ef.createNome(pilota), ef.createNumero(0)));
			}
			if (posizioneArrivo < punteggio.length)
				c.getPunti().setValue(c.getPunti().wIntValue() + punteggio[posizioneArrivo++]);
		}
	}

	private static class StampaClassificheVisitor extends MotoMondialeTraverseAllVisitor {
		private PrintStream out;
		private Map<String, Pilota> mappaPiloti;
		public StampaClassificheVisitor(PrintStream out) {
			this.out = out;
		}

		@Override
		public void visit(MotoMondiale entity) {
			mappaPiloti = DerivaMappaPilotiVisitor.getMappaPiloti(entity);
			entity.getClassifiche().accept(this);
		}

		@Override
		public void visit(Classifica entity) {
			out.println("Classifica: "+entity.getNome().getValue());
			out.println("Pilota\t\tPunti\tMoto");
			entity.getClassificati().accept(this);
			out.println();
		}
		
		@Override
		public void visit(Classificato entity) {
			String nomePilota = entity.getNome().getValue();
			out.print(nomePilota+"\t"+entity.getPunti().getValue());
			Pilota pilota = mappaPiloti.get(nomePilota);
			if (pilota != null)
				out.println("\t"+((Team) pilota.wGetParent()).getCostruttore());
			else
				out.println();
		}
	}

	private static class ValidaPilotiVisitor extends MotoMondialeTraverseAllVisitor {
		private Map<String, Pilota> mappaPiloti;
		private Set<String> pilotiNonCensiti = new HashSet<String>();
		private PrintStream out;
		public ValidaPilotiVisitor(Map<String, Pilota> mappaPiloti, PrintStream out) {
			this.mappaPiloti = mappaPiloti;
			this.out = out;
		}

		@Override
		public void visit(MotoMondiale entity) {
			entity.getCalendario().accept(this);
		}
		@Override
		public void visit(Gara entity) {
			entity.getOrdineDArrivo().accept(this);
		}

		@Override
		public void visit(Nome entity) {
			String nomePilota = entity.wStringValue();
			if (!mappaPiloti.containsKey(nomePilota) && pilotiNonCensiti.add(nomePilota))
				out.println("Pilota non appartenente a nessun team: "+nomePilota);
		}
	}
	
	private static class DerivaMappaPilotiVisitor extends MotoMondialeTraverseAllVisitor {
		private Map<String, Pilota> mappaPiloti = new HashMap<String, Pilota>();
		
		public static Map<String, Pilota> getMappaPiloti(IMotoMondialeEntity mm) {
			DerivaMappaPilotiVisitor dmp = new DerivaMappaPilotiVisitor();
			mm.accept(dmp);
			return dmp.mappaPiloti;
		}		
				
		@Override
		public void visit(Pilota entity) {
			mappaPiloti.put(entity.getNome().getValue(), entity);
		}
	}
}
